;*********************************************************************
; COMMODORE VIC 20 BOOT USING BASIC 2.0
; written by Robert Hurst <robert@hurst-ri.us>
; updated version: 30-Oct-2011
; Modified March 2023 to assemble using CBM prg Studio 4.0.1
;
;                .fileopt author,        "your name"
;        .fileopt comment,       "your program name"
;        .fileopt compiler,      "VIC 20 ASSEMBLER"

;                .include "VIC-SSS-MMX.h"

;*********************************************************************
; Commodore BASIC 2.0 program
;
; LOAD "YOUR PROGRAM.PRG",8
; RUN
;
;                .segment "BASIC"

;                .word   RUN             ; load address
;RUN:    .word   @end    ; next line link
;                .word   2010    ; line number
;                .byte   $9E             ; BASIC token: SYS
;                .byte   <(MAIN / 1000 .mod 10) + $30
;                .byte   <(MAIN / 100 .mod 10) + $30
;                .byte   <(MAIN / 10 .mod 10) + $30
;                .byte   <(MAIN / 1 .mod 10) + $30
;                .byte   0               ; end of line
;@end:   .word   0               ; end of program
;

;replaced with Generate SYS Call tool output

; 10 SYS (4640)

*=$1201

        BYTE    $0E, $12, $0A, $00, $9E, $20, $28,  $34, $36, $34, $30, $29, $00, $00, $00


;*********************************************************************
; Starting entry point for this program
;
;                .segment "STARTUP"
*=$1220

MAIN 
                LDX $FFFC
                LDY $FFFD
                STX $0318
                STY $0319               ; enable RESTORE key as RESET
                LDA MACHINE
                CMP #$05
                BNE PALF
                ;
                ; NTSC setup
NTSC            LDX #<NTSCT              ; load the timer low-byte latches
                LDY #>NTSCT
                LDA #$69                ; raster line 210/211
                BNE IRQSYNCF
NTSCt = $4243                   ; (261 * 65 - 2)
                ;
                ; PAL setup
PALF            LDX #<PALT              ; load the timer low-byte latches
                LDY #>PALT
                LDA #$76                ; raster line 228/229
PALT = $5686                    ; (312 * 71 - 2)
                ;
IRQSYNCF 
                CMP VIC+$04
                BNE IRQSYNCF
                STX $9126               ; load T1 latch low
                STY $9125               ; load T1 latch high, and transfer both to T1 counter


;*********************************************************************
; Now that all the VIC startup initialization stuff is completed,
; you can append one-time startup code/data here, i.e., like a splash
; title screen.  Then, you must jump to your CODE segment, linked
; outside of VIC's internal RAM address space ...
;
RUNONCE 
                ;
                ;====  INSERT ANY ADDITIONAL STARTUP CODE HERE  ====
                ;
                JMP RESTART


;*********************************************************************
; VIC Software Sprite Stack 2010 (VIC-SSS-MMX)
;
; The above BASIC loader will be overwritten by SSS upon its
; initialization (SSSINIT).  The linker will fill this reserved space
; with values used for the dual video frame buffers, play field, and
; the sprite image buffers and registers: 4096 - 6207 ($1000 - $1BFF)
;
; $1000 - $11FF         VICFRAME1 - first video buffer
; $1200 - $13FF         VICFRAME2 - second video buffer
; $1400 - $15FF         PLAYFIELD - write-pending screen buffer
; $1600 - $17FF         PLAYCOLOR - write-pending color / dirty buffer
; $1800 - $1BFF         Sprite image buffers & registers
;
;                        .segment "SSSBUF"

*=$1800
SSSBUF   bytes 512            ; (64*8) this can be resized as required --
                              ; if all 64-chars are used by sprites, that
                              ; exhausts all 128 custom characters for
                              ; double-buffering (x2)
;
; SPRITE REGISTERS (17)
;
SPRITEBACK     bytes SPRITEMAX  ; 1st char this sprite is in collision with
SPRITEBUFH     bytes SPRITEMAX  ; pointer within sprite image buffer
SPRITEBUFL     bytes SPRITEMAX  ; pointer within sprite image buffer
SPRITEC1H      bytes SPRITEMAX  ; pointer within sprite display character pool
SPRITEC1L      bytes SPRITEMAX  ; pointer within sprite display character pool
SPRITEC2H      bytes SPRITEMAX  ; pointer within sprite display character pool
SPRITEC2L      bytes SPRITEMAX  ; pointer within sprite display character pool
SPRITECOL      bytes SPRITEMAX  ; 4-bit VIC color code
SPRITECX       bytes SPRITEMAX  ; sprite collision X-coord
SPRITECY       bytes SPRITEMAX  ; sprite collision Y-coord
SPRITEDEF      bytes SPRITEMAX  ; function/matrix definition (see explanation below)
SPRITEH        bytes SPRITEMAX  ; number of raster lines (1-16)
SPRITEIMGH     bytes SPRITEMAX  ; pointer to source graphic for rendering at 0,0
SPRITEIMGL     bytes SPRITEMAX  ; pointer to source graphic for rendering at 0,0
SPRITEX        bytes SPRITEMAX  ; horizontal pixel coordinate, visible >0 - <SSSCLIPX
SPRITEY        bytes SPRITEMAX  ; vertical pixel coordinate, visible >0 - <SSSCLIPY
SPRITEZ        bytes SPRITEMAX  ; bit 0: last rendered (0 = SPRITEC1; 1 = SPRITEC2)
                                                        ; bit 1: fast copy (0 = merge; 1 = copy)
                                                        ; bit 3: sprite collision
                                                        ; bit 4: sprite image is clipped by a static cell
                                                        ; bit 5: background is all SSSNULLs
                                                        ; bit 6: copy/merge into alternate sprite character pool
                                                        ; bit 7: copy/shift sprite image into its buffer
;
; SPRITEDEF is a bit-structure of these characteristics:
; - height              bit 0: 0 = 8px; 1 = 16px
; - width               bit 1: 0 = 8px; 1 = 16px
; - float Y             bit 2: flag: 0=fixed cell, 1=vertical float
; - float X             bit 3: flag: 0=fixed cell, 1=horizontal float
; - repeat              bit 4: flag: 0=independent, 1=same as previous
; - ghost               bit 5: flag: 0=merge image, 1=invert image
; - collision   bit 6: flag: 0=fast copy, 1=detect
; - enabled             bit 7: flag: 0=invisible, 1=visible
;
; SSS runtime variables:

*=$1b10

sss             bytes 48      ; screen row index, computed from PLAYCOLS in SSSINIT
;
; other initialized data can be appended here:
;
;                       .segment "RODATA"
;moved to MMX module


;*********************************************************************
; VIC Custom Graphic Characters
;
; If < 64 will be used for the software sprite stack, the remaining
; unused characters can be used for other custom graphics, beginning
; at $1C00 where "@", "A", "B", ... characters can be redefined.
;
; Do not use this as an initialized segment if you plan on linking
; this source as a future game cartridge later.  You must COPY any
; read-only data into this address space.
;
; If your data was saved from some tool in binary format, you can
; include that binary file here as:
;               .incbin "graphics.dat"
;
; else, just enter each 8x8 values here, such as:
;               .byte   $FF,$81,$81,$81,$81,$81,$81,$FF
; or:
;               .byte   %11111111       ; square
;               .byte   %10000001
;               .byte   %10000001
;               .byte   %10000001
;               .byte   %10000001
;               .byte   %10000001
;               .byte   %10000001
;               .byte   %11111111
;
;               .segment "MYCHAR"

*=$1c00
;*********************************************************************
; Your main program code starts here
;
;               .segment "CODE"

*=$2000
RESTART

incasm "demo.asm"
incasm "VIC-SSS-MMX.asm"
