;*********************************************************************
; DEMO featuring VIC Software Sprite Stack - MMX edition
; written by Robert Hurst <spock@hurst-ri.us>
; updated version: 19-Apr-2010
; Modified March 2023 to assemble using CBM prg Studio.
; Demo8 added to illustrate multicolour sprites and a scrolling 
; background simulation.
; Note that text strings need to be in the reversed character set. 
; This project uses a custom character map (see Tools/Options/Assembler)
; to override the standard mapping.
;*********************************************************************

;                .include "VIC-SSS-MMX.h"
;                .include "SSS.h"

;                .segment "CODE"

                LDA #$1b
                STA VIC+$0F
                LDA #6
                STA COLORCODE
                JSR SSSINIT             ; must occur BEFORE re-directing IRQ
;
; implement my interrupt vector
                SEI
                LDX #<SSSIRQ
                LDY #>SSSIRQ
                STX $0314
                STY $0315
                CLI

                ; set up running man sprite (Hero) sprite 0
                ; no shape definition yet
                LDA #%10001101  ; enable an 8x16 sprite that floats X-Y
                LDY #16                 ; pixel height
                JSR SSSCREATE
                LDX #0          ;Column
                STX R0          ;initialise change direction flag
                LDY #112        ;Row (8*12)+16 
                JSR SSSMOVEXY

                ; set up small chomper sprite sprite 1
                ; no shape definition yet
                LDA #%10001100  ; enable an 8x8 sprite that floats X-Y
                LDY #8                  ; pixel height
                JSR SSSCREATE
                LDA SSSCLIPX    ; column: position just on screen
                CLC
                ADC #8
                TAX             ; column parameter
                LDY #92         ; Row: (8*9)+20
                JSR SSSMOVEXY   

          
                ; set up big chomper sprite 2
                ; no shape definition yet  
                LDA #%10001011  ; enable an 16x16 sprite that floats X
                LDY #16                 ; pixel height
                JSR SSSCREATE
                LDX #0          ; Column
                STX R1          ; initialise change direction flag
                LDY #136        ;Row: (8*15)+16
                JSR SSSMOVEXY

TITLE           ;intro screen
                JSR SSSPRINTS  ;print the following text using colours $F0 - F7
                               ;note text specified using Reverse character codes
                               ;13 is carriage return. String terminated by 0
                byte $F2,"SOFTWARE ",$F5,"SPRITE ",$F6,"STACK",13,13,0
                JSR SSSPRINTS
                byte $F0,"  A TUTORIAL DEMO",13,0
                JSR SSSPRINTS
                byte $F4,"  BY ROBERT HURST",13,13,0
                JSR SSSPRINTS
                byte $F3," RELEASED 19-APR-2010"
                byte 0
@loop           JSR HEROMOVE ;Get running man going
                JSR QMANMOVE ;Get chomper going
                JSR ANYKEY   ;wait for user input also uses SSSinit to clear sprites
                BEQ @loop

DEMO1 
                JSR SSSPRINTS
                byte $F4,"THE FIRST THING TO DO",13
                byte "IS TO INITIALIZE THE",13
                byte "SOFTWARE SPRITE STACK:",13,13,0
                JSR SSSPRINTS
                byte $F6,"JSR ",$F2,"SSSINIT",13,13,13
                byte $F0,"IT PUTS THE VIC IN A",13
                byte "DUAL-BUFFER VIDEO MODE"
                byte $F6,"  @ $1000 & $1200",13
                byte $F0,"A PLAYFIELD AND COLOR",13
                byte $F6,"  @ $1400 & $1600",13
                byte $F0,"REGISTERS AND STORAGE",13
                byte $F6,"  @ $1800 - $1B7F",13
                byte $F0,"USES CUSTOM CHARACTERS"
                byte $F6,"  @ $1C00 - $1FFF"
                byte 0
@loop           JSR ANYKEY
                BEQ @loop

DEMO2 
                JSR SSSPRINTS
                byte $F4,"WRITING TO THE",13
                byte "PLAYFIELD:",13,13
                byte $F6,"LDY #",$F5,"4",13
                byte $F6,"LDX #",$F5,"18",13
                byte $F6,"JSR ",$F2,"SSSPLOT",13
                byte $F6,"JSR ",$F2,"SSSPRINTS",13
                byte $F0,"BYTE ",$F5,$a2,"ABC",$a2,",0",13,13
                byte $F6,"LDA #",$F5,"6",13
                byte $F6,"STA ",$F4,"COLORCODE",13
                byte $F6,"LDY #",$F5,"6",13
                byte $F6,"LDX #",$F5,"19",13
                byte $F6,"JSR ",$F2,"SSSPLOT",13
                byte $F6,"LDA #",$F5,"$84",13
                byte $F6,"JSR ",$F2,"SSSPOKE",13,13
                byte $F6,"LDY #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSFLIP",13
                byte 0
                LDY #4     ;Row
                LDX #18    ;Column
                JSR SSSPLOT   ;Plot cursor
                JSR SSSPRINTS ;Print string at cursor position
                BYTE "ABC",0
                LDY #6
                LDX #19
                JSR SSSPLOT    ;Move cursor
                LDA #6
                STA COLORCODE  ;Set colour
                LDA #$84
                JSR SSSPOKE    ;Display character 'D'
@loop           JSR ANYKEY
                BEQ @loop

DEMO3 
                JSR SSSPRINTS
                byte $F4,"MAKING A SIMPLE",13
                byte "SPRITE (",$F2,"RED HEART",$F4,"):",13,13
                byte $F6,"LDA #",$F5,"%10001100",13
                byte $F6,"LDY #",$F5,"8",13
                byte $F6,"JSR ",$F2,"SSSCREATE",13,13
                byte $F6,"LDA #",$F5,"2",13
                byte $F6,"LDX #",$F5,"$98",13
                byte $F6,"LDY #",$F5,"$82",13
                byte $F6,"JSR ",$F2,"SSSANIM",13,13
                byte $F6,"LDX #",$F5,"160",13
                byte $F6,"LDY #",$F5,"96",13
                byte $F6,"JSR ",$F2,"SSSMOVEXY",13,13
                byte $F6,"LDY #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSFLIP"
                byte 0
                JSR HEART   ; set up a sprite using a ROM character
@loop           JSR ANYKEY
                BEQ @loop

DEMO4 
                JSR SSSPRINTS
                byte $F4,"MOVING A SIMPLE",13
                byte "SPRITE:",13,13
                byte $F0,"@LOOP:",13
                byte $F6,"LDX #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSUSE",13,13
                byte $F6,"LDX ",$F4,"SPRITEX",13
                byte $F6,"INC ",$F4,"SPRITEY",13
                byte $F6,"LDY ",$F4,"SPRITEY",13
                byte $F6,"JSR ",$F2,"SSSMOVEXY",13,13
                byte $F6,"LDY #",$F5,"1",13
                byte $F6,"JSR ",$F2,"SSSFLIP",13
                byte $F6,"JMP ",$F0,"@LOOP",13
                byte 0
                JSR HEART  ;reinstate the HEART sprite (0)
@loop           LDX #0
                JSR SSSUSE   ;use sprite 0
                LDX SPRITEX
                INC SPRITEY  ;amount of increment controls speed
                LDY SPRITEY
                JSR SSSMOVEXY  ;move sprite around vertically
                JSR ANYKEY
                BEQ @loop

DEMO5 
                JSR SSSPRINTS
                byte $F4,"STATIC CELLS ON",13
                byte "THE PLAYFIELD:",13,13
                byte $F6,"LDY #",$F5,"10",13
                byte $F6,"LDX #",$F5,"18",13
                byte $F6,"JSR ",$F2,"SSSPLOT",13
                byte $F6,"LDA #",$F5,"6",13
                byte $F6,"STA ",$F4,"COLORCODE",13
                byte $F6,"LDA #",$F5,"128+81",13
                byte $F6,"JSR ",$F2,"SSSCELL",13
                byte $F6,"LDY #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSFLIP",13
                byte $F6,"LDY #",$F5,"10",13
                byte $F6,"LDX #",$F5,"18",13
                byte $F6,"JSR ",$F2,"SSSPLOT",13
                byte $F6,"LDA #",$F5,"3",13
                byte $F6,"STA ",$F4,"COLORCODE",13
                byte $F6,"LDA #",$F5,"128+87",13
                byte $F6,"JSR ",$F2,"SSSCELL",13
                byte $F6,"LDY #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSFLIP",13
                byte 0
                ;
                JSR HEART  ; reinstate HEART sprite (0)
                JSR BALL   ; set up a static character cell
                ;
@loop           LDX #0
                JSR SSSUSE
                LDX SPRITEX
                INC SPRITEY
                LDY SPRITEY
                JSR SSSMOVEXY
                JSR ANYKEY
                BEQ @loop

DEMO6 
                JSR SSSPRINTS
                byte $F4,"COLLISION",13
                byte "DETECTION:",13,13
                byte $F6,"LDA ",$F4,"SPRITEDEF",13
                byte $F6,"ORA #",$F5,"$40",13
                byte $F6,"STA ",$F4,"SPRITEDEF",13
                byte $F7,"... MOVE SPRITE",13
                byte $F6,"LDY #",$F5,"0",13
                byte $F6,"JSR ",$F2,"SSSFLIP",13,13
                byte $F6,"LDY #",$F5,"27",13
                byte $F6,"LDA ",$F4,"SPRITEZ",13
                byte $F6,"AND #",$F5,"$08",13
                byte $F6,"BEQ ",$F0,"@CONT",13
                byte $F6,"DEY",13
                byte $F0,"@CONT:",13
                byte $F6,"STY ",$F5,"$900F",13,13,13
                byte $F4,"SPRITEBACK",$F6," = "
                byte 0
                LDX CRSRCOL  ;save cursor position
                STX R3
                LDY CRSRROW
                STY R4
                ;
                LDY #4
                LDX #18
                JSR SSSPLOT   ;position cursor
                JSR SSSPRINTS
                BYTE "ABC",0  ;print this text enclosed in quotes
                JSR HEART     ;reinstate the heart sprite
                JSR BALL      ;reinstate the static cell
                LDA SPRITEDEF ;enable collision detection for sprite 0
                ORA #$40
                STA SPRITEDEF
                ;
@loop           LDX #0
                JSR SSSUSE
                LDX SPRITEX
                INC SPRITEY
                LDY SPRITEY
                JSR SSSMOVEXY ;move the sprite
                JSR ANYKEY
                BNE DEMO7     ;quit on key press
                LDY #27
                LDA SPRITEZ   ;collision register
                AND #$08      ;mask bit 3
                BEQ @cont     ;no collision registered
                DEY           ;collision so flash border
@cont           STY $900F
                LDX R3        ;recover cursor position
                LDY R4
                JSR SSSPLOT   
                LDA #0
                STA COLORCODE  ;set plot colour as black
                LDA SPRITEBACK ;get the ID of collision character or space
                JSR SSSPRINT   ;print the character
                JMP @loop      ; repeat until key pressed

DEMO7 
                LDA #30
                STA $900F
                JSR SSSPRINTS
                byte $F4,"GHOST MODE:",13,13
                byte $F6,"LDA ",$F4,"SPRITEDEF",13
                byte $F6,"ORA #",$F5,"$20",13
                byte $F6,"STA ",$F4,"SPRITEDEF",13
                byte 0
                LDX #7
                LDA #$FF
@fill           STA $1C08,X  ;set up a solid block in user character space
                DEX
                BPL @fill
                ;
                LDA #%11101111  ; 16x16 sprite floats X-Y and uses Ghosting (bit 5)
                LDY #$10                ; height
                JSR SSSCREATE
                LDA #$0a                ; red
                LDX #<BIGRED    ;create a new sprite coloured red
                LDY #>BIGRED
                JSR SSSANIM
                LDX #0          ;X position for sprite
                LDA SSSCLIPY    ;use Y clip value to form Y position for sprite
                LSR
                TAY
                JSR SSSMOVEXY  
                ;
                LDY #9       ;set up a position record in user registers
                LDX #2
                STY R3
                STX R4
@bar1
                LDX R4
                JSR SSSPLOT   ;plot start
                JSR SSSPRINTS ;print the following pattern across the screen
                byte $F5,1,1,1,SSSNULL,SSSNULL,SSSNULL
                byte $F6,1,1,1,SSSNULL,SSSNULL,SSSNULL
                byte $F7,1,1,1,SSSNULL,SSSNULL,SSSNULL
                byte 0
                INC R3
                LDY R3
                CPY #14   ;quit when 5 rows are printed
                BNE @bar1

@loop           LDX #0
                JSR SSSUSE
                INC SPRITEX
                LDX SPRITEX
                LDY SPRITEY
                JSR SSSMOVEXY  ;move the BigRed sprite across the screen
                JSR ANYKEY     ;until a key press
                BEQ @loop


;multicolour sprite test

DEMO8   
                lda #$1b
                sta $900f
                lda #%10001101  ; 8x16 sprite 
                ldy #14         ; height
                jsr SSSCREATE
                stx R1
                lda #$0f           ;multicolour with yellow character colour
                ldx #<carsprite    ;create a new sprite 
                ldy #>carsprite
                jsr SSSANIM
                lda SSSCLIPX
                lsr
                tax             ;X position for sprite
                lda SSSCLIPY    ;use Y clip value to form Y position for sprite
                lsr
                clc
                adc #12
                tay
                jsr SSSMOVEXY  

                lda #%10001101  ; 8x16 sprite 
                ldy #$16        ; height
                jsr SSSCREATE
                lda #$0a             ;multicolour with red character colour
                ldx #<lorrysprite    ;create a new sprite coloured red
                ldy #>lorrysprite
                jsr SSSANIM
                lda SSSCLIPX
                lsr
                tax             ;X position for sprite
                lda SSSCLIPY    ;use Y clip value to form Y position for sprite
                lsr
                tay
                jsr SSSMOVEXY  

                lda #%10001100  ; 8x8 sprite 
                ldy #$08        ; height
                jsr SSSCREATE
                lda #$0a           ;multicolour with red character colour
                ldx #<lorrytail    ;create a new sprite 
                ldy #>lorrytail
                jsr SSSANIM
                lda SSSCLIPX
                lsr
                tax             ;X position for sprite
                lda SSSCLIPY    ;use Y clip value to form Y position for sprite
                lsr
                clc
                adc #16
                tay
                jsr SSSMOVEXY

                ldy #0            ;display lorry and car (hidden on lorry)
                jsr SSSFLIP

                lda #32
                sta R0           ;counter for moving the car out of the lorry
@loop           ldx R1
                jsr SSSUSE       ;refer to the car sprite
  
                ldx R1           ;point to the right coordinate registers
                ldy SPRITEY,x
                iny
                lda SPRITEX,x
                tax
                jsr SSSMOVEXY    ;move the car down
                ldx #1           ;refresh the lorry display
                jsr SSSUSE
                jsr SSSTOUCH
                ldx #2           ;refresh the lorry tail display
                jsr SSSUSE
                jsr SSSTOUCH
                ldy #2
                jsr SSSFLIP      ;use the display frame display
                dec R0
@stop           bne @loop

;scrolling road part
;remove lorry 
;draw road 
;scroll it 

                ;base sprite which will be copied 11 times to give 6 blocks
                ;each side to mark the edges of the road
road            lda #%10000101  ; 8x16 sprite 
                ldy #16         ; height
                jsr SSSCREATE
                lda #$00             
                ldx #<edgesprite    ;create a new sprite coloured black
                ldy #>edgesprite
                jsr SSSANIM
                ldy #0           ;Y sprite position
                sty edgeY
                ldx #48
                stx edgeX        ;X sprite position
                jsr SSSMOVEXY  

@loop           lda #%10000101  ; new sprite
                ldy #16         ; height
                jsr SSSCREATE  
                lda #$00             
                ldx #<edgesprite    
                ldy #>edgesprite
                jsr SSSANIM     ;create a new sprite coloured black
                ldy edgeY       ; Y position in Y
                lda edgeX
                eor #$a8        ; toggle between left and right edge
                sta edgeX
                tax             ; X position in X
                jsr SSSMOVEXY   
                lda edgeX
                cmp #152        ; ? just done right edge
                bcc @skip
                lda edgeY       ; set next Y position
                clc
                adc #51
                sta edgeY
@skip           lda edgeY       
                cmp #255         ; ? have we created all 10 edge sprites 
                bcc @loop
 
                ldx #0           ; delete the lorry
                stx SPRITEDEF+1
                stx SPRITEDEF+2
                ldy #0           ; update display
                jsr SSSFLIP   
              
; scroll road
                lda #0
                sta DEMODELAY

@loop1          ldx #3       ;starting edge sprite
@loop2          stx R0
                jsr SSSUSE
                lda SPRITEY,x ;get the current Y
                tay
                iny
                iny           ;increment Y by 2
                lda SPRITEX,x ;get the current X
                tax
                jsr SSSMOVEXY  ;update on next flip
                ldx R0
                inx           ; index next sprite
                cpx #13       ; ? have we done all 10
                bne @loop2
                jsr ANYKEY    ; check for a cancelling key press
                beq @loop1



FINI           JMP RESET               ; bye-bye!

ANYKEY 
                LDY PLAYROWS
                DEY
                LDX #0
                JSR SSSPLOT
                JSR SSSPRINTS
                byte $F6, "-=: PRESS ANY KEY :=-"
                byte 0
                ;
@user           LDY DEMODELAY
                JSR SSSFLIP
                LDA $C5
                CMP #$40
                BEQ @repeat             ; keypress?
@wait           LDA $C5
                CMP #$40
                BNE @wait               ; release key
                JSR SSSINIT             ; start over - clears all sprite data
                LDA #1
@repeat 
                RTS

                ;set up a protected character cell. These are not changed by contact
                ; with a moving sprite
BALL            
                LDY #10
                LDX #18
                JSR SSSPLOT   ;set charcter position
                LDA #4
                STA COLORCODE ;set colour red
                LDA #128+81
                JSR SSSCELL   ;write character 81 filled circle
                LDY #0
                JSR SSSFLIP   ;immediate update
                ;
                LDY #10
                LDX #18
                JSR SSSPLOT
                LDA #5
                STA COLORCODE ;set colour blue
                LDA #128+87   ;write character 87 open circle
                JSR SSSCELL
                ;
                RTS

HEART 
                LDA #%10001100  ; enable 8X8 float
                LDY #8          ; height
                JSR SSSCREATE
                ; RED HEART
                LDA #2    ; column
                LDX #$98  ;
                LDY #$82  ; ROM address 8298 = char $53
                JSR SSSANIM
                LDX #160  ; column
                LDY #96   ; row in sprite pixels
                JSR SSSMOVEXY
                LDY #0
                JSR SSSFLIP ; display now
                RTS

HEROMOVE 
                LDX #0
                STX sssNUM   ;specify what sprite we are using
                LDA #5       ;sprite colour param
                LDX #<HERO   ;address of sprite data for frame 1
                LDY #>HERO
                JSR SSSANIM  
                LDA SPRITEX  ;get sprite X for sprite 0
                LSR          ;divide by 2
                AND #$03     ;convert to frame (0-3)
                BEQ @ck0     ; check position
                CMP #$02
                BEQ @r2      ; frame 3
@r1 
                LDX #<HERO16  ;frame 2
                LDY #>HERO16
                STX SPRITEIMGL ;directly update image source
                STY SPRITEIMGH
                BNE @ck0       ;unconditional jump to check position
@r2
                LDX #<HERO32  ;frame 3
                LDY #>HERO32
                STX SPRITEIMGL
                STY SPRITEIMGH
@ck0                                   ; process RIGHT
                LDA R0        ;SS user register holding change direction flag
                CMP #-1       ; signal to change direction
                BEQ @ck2      ; change direction
                LDA SPRITEX   ; get the X position
                SEC
                SBC #8        ; reduce by character width
                CMP SSSCLIPX  ; ? clipped
                BEQ @ck1      ; handle clip
                INC SPRITEX   ; move right
                BNE @fini     
@ck1            LDA #-1       ; signal change direction
                STA R0
@ck2                                   ; process LEFT
                LDA R0
                AND #%00000100   ;curious, R0 is either 0 or -1
                BEQ @ck3
                LDA SPRITEIMGL  ; Point to next animation frame data block
                CLC
                ADC #$30
                BCC @lcc
                INC SPRITEIMGH
@lcc            STA SPRITEIMGL  
                LDA SPRITEX     ; 
                BEQ @ck3
                DEC SPRITEX
@fini           RTS
@ck3            LDA #0       ;clear change direction flag
                STA R0
                RTS

                ;complete the display of the chomper sprite
QMANMOVE        ; user register R1 to signal size and direction change
                LDA R1
                CMP #-1
                BEQ @big
                LDX #1       ;indicate we are updating sprite 1
                STX sssNUM
                LDX #<QBALL  ;use this data for the sprite
                LDY #>QBALL
                DEC SPRITEX+1 ;decrement the X position for sprite 1
                BEQ @fini1    ;branch if the X is zero
                LDA SPRITEX+1 ;change the image every 2 pixel change
                AND #$02
                BEQ @ck1
                LDX #<QBALL2  ;alternative image
                LDY #>QBALL2
@ck1            LDA #7        ;colour yellow
                JSR SSSANIM
                RTS
@fini1          LDA #-1       ;reached the edge so change direction
                STA R1        ;
                LDA SSSCLIPX  ;use the system clip position
                CLC
                ADC #8
                STA SPRITEX+1 ;to set the starting X position
                RTS

                ; change sprite and start moving right
@big            ; updating sprite 2
                LDX #2
                STX sssNUM
                LDY #>QUIKMAN  ; get data high value
                INC SPRITEX+2  ; increment X for sprite 2
                LDA SPRITEX+2
                CMP SSSCLIPX   ; ? X = clip point
                BEQ @fini2     ; change direction
                AND #$03       ; use position to decide which data set to use
                ASL
                ASL
                ASL
                ASL
                ASL
                ADC #<QUIKMAN
                BCC @cc
                INY           ; data high address
@cc             TAX           ; data low address
                LDA #7        ; colour yellow
                JSR SSSANIM
                RTS
@fini2          LDA #0        ; signal direction change
                STA R1
                STA SPRITEX+2 ; reset X position
                RTS

;                .segment "RODATA"


DEMODELAY       byte 1  ;Frame delay for demo 1 - 7
edgeX           byte 0
edgeY           byte 0


BIGRED 
                byte   %00001111
                byte   %00111111
                byte   %01111111
                byte   %11111111
                byte   %11100111
                byte   %11000011
                byte   %11100111
                byte   %11111111

                byte   %11111111
                byte   %11111111
                byte   %11110011
                byte   %11001100
                byte   %11111111
                byte   %11111111
                byte   %11001100
                byte   %10001000

                byte   %11110000
                byte   %11111100
                byte   %11111110
                byte   %11111111
                byte   %11100111
                byte   %11000011
                byte   %11100111
                byte   %11111111

                byte   %11111111
                byte   %11111111
                byte   %11001111
                byte   %00110011
                byte   %11111111
                byte   %11111111
                byte   %00110011
                byte   %00010001

HERO 
                ; moving right frame #0 (& standing)
                byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %01011010
                byte   %01011010
                byte   %01011010
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011100
                byte   %00010000
                ; moving right frame #1 / #3
HERO16          byte   %00000000
                byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %01011100
                byte   %01011100
                byte   %00111110
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00010100
                byte   %11110010
                byte   %10000010
                byte   %00000010
                byte   %00000011
                ; moving right frame #2
HERO32          byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %01011010
                byte   %10011001
                byte   %01011000
                byte   %00011000
                byte   %00011000
                byte   %00100100
                byte   %00100010
                byte   %01000001
                byte   %01000001
                byte   %10000001
                byte   %10000001
                byte   %00000000
OREH 
                ; moving left frame #0
                byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %01011010
                byte   %01011010
                byte   %01011010
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00111000
                byte   %00001000
                ; moving left frame #1 / #3
                byte   %00000000
                byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %00111010
                byte   %00111010
                byte   %01111100
                byte   %00011000
                byte   %00011000
                byte   %00011000
                byte   %00101000
                byte   %01001111
                byte   %01000001
                byte   %01000000
                byte   %11000000
                ; moving left frame #2
                byte   %00011000
                byte   %00011000
                byte   %00000000
                byte   %00111100
                byte   %01011010
                byte   %10011001
                byte   %00011010
                byte   %00011000
                byte   %00011000
                byte   %00100100
                byte   %01000100
                byte   %10000010
                byte   %10000010
                byte   %10000001
                byte   %10000001
                byte   %00000000

QBALL          byte   $3C, $7E, $FF, $FF, $FF, $FF, $7E, $3C  
QBALL2         byte   $7C, $3E, $1F, $0F, $0F, $1F, $3E, $7C

QUIKMAN 
                byte   %00000111
                byte   %00011111
                byte   %00111111
                byte   %01111111
                byte   %01111111
                byte   %11111111
                byte   %11111111
                byte   %11111110
                ;
                byte   %11111110
                byte   %11111111
                byte   %11111111
                byte   %01111111
                byte   %01111111
                byte   %00111111
                byte   %00011111
                byte   %00000111
                ;
                byte   %11100000
                byte   %11111000
                byte   %11110000
                byte   %11100000
                byte   %11000000
                byte   %10000000
                byte   %00000000
                byte   %00000000
                ;
                byte   %00000000
                byte   %00000000
                byte   %10000000
                byte   %11000000
                byte   %11100000
                byte   %11110000
                byte   %11111000
                byte   %11100000
                ; QUICKMAN2
                byte   %00000111
                byte   %00011111
                byte   %00111111
                byte   %01111111
                byte   %01111111
                byte   %11111111
                byte   %11111111
                byte   %11111111
                ;
                byte   %11111111
                byte   %11111111
                byte   %11111111
                byte   %01111111
                byte   %01111111
                byte   %00111111
                byte   %00011111
                byte   %00000111
                ;
                byte   %11100000
                byte   %11111000
                byte   %11111100
                byte   %11111110
                byte   %11111100
                byte   %11110000
                byte   %11000000
                byte   %00000000
                ;
                byte   %00000000
                byte   %11000000
                byte   %11110000
                byte   %11111100
                byte   %11111110
                byte   %11111100
                byte   %11111000
                byte   %11100000
                ; QUICKMAN3
                byte   %00000111
                byte   %00011111
                byte   %00111111
                byte   %01111111
                byte   %01111111
                byte   %11111111
                byte   %11111111
                byte   %11111111
                ;
                byte   %11111111
                byte   %11111111
                byte   %11111111
                byte   %01111111
                byte   %01111111
                byte   %00111111
                byte   %00011111
                byte   %00000111
                ;
                byte   %11100000
                byte   %11111000
                byte   %11111100
                byte   %11111110
                byte   %11111110
                byte   %11111111
                byte   %11111111
                byte   %11111111
                ;
                byte   %11111111
                byte   %11111111
                byte   %11111111
                byte   %11111110
                byte   %11111110
                byte   %11111100
                byte   %11111000
                byte   %11100000
                ; QUICKMAN4
                byte   %00000111
                byte   %00011111
                byte   %00111111
                byte   %01111111
                byte   %01111111
                byte   %11111111
                byte   %11111111
                byte   %11111111
                ;
                byte   %11111111
                byte   %11111111
                byte   %11111111
                byte   %01111111
                byte   %01111111
                byte   %00111111
                byte   %00011111
                byte   %00000111
                ;
                byte   %11100000
                byte   %11111000
                byte   %11111100
                byte   %11111110
                byte   %11111100
                byte   %11110000
                byte   %11000000
                byte   %00000000
                ;
                byte   %00000000
                byte   %11000000
                byte   %11110000
                byte   %11111100
                byte   %11111110
                byte   %11111100
                byte   %11111000
                byte   %11100000


edgesprite       BYTE    $f0,$f0,$f0,$f0,$f0,$f0,$f0,$f0
                 BYTE    $f0,$f0,$f0,$f0,$f0,$f0,$f0,$f0

carsprite        BYTE    $3C,$D7,$D7,$EB,$69,$69,$69,$69
                 BYTE    $69,$69,$69,$EB,$D7,$3C,$00,$00

lorrysprite      BYTE    $3C,$D7,$D7,$EB,$69,$69,$69,$69
                 BYTE    $FF,$69,$69,$69,$69,$69,$69,$69
lorrytail        BYTE    $69,$69,$69,$69,$EB,$EB,$EB,$3C

